#  Copyright (C) 2006-2007 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

begin
	require 'Qt4'
rescue LoadError
	require 'Qt'
	raise LoadError.new( 'Wrong Qt version found (Qt4 expected but Qt3 found)' ) if ( ! defined?( Qt::StringListModel ) )
end
require 'date'
require File.expand_path( File.dirname( __FILE__ ) + '/../i18n/i18n' )
$KCODE='u' # unicode support

module QT4

	def QT4.empty_string?( text )
		return text ? (text.size == 0 ? true : text.strip().empty?() ) : true
	end

	class BaseDialog < Qt::Dialog

		attr_reader :values

		def initialize( values )
			super( nil )
			@values = values
		end

		def create_action_buttons( mode='right', accept_text=nil, cancel_text= nil )
			accept_text = QT4.empty_string?( accept_text ) ? I18N.get( 'gui.common.accept' ) : accept_text
			cancel_text = QT4.empty_string?( cancel_text ) ? I18N.get( 'gui.common.cancel' ) : cancel_text
			@accept_button = Qt::PushButton.new( accept_text, self )
			@cancel_button = Qt::PushButton.new( cancel_text, self )
			connect( @accept_button, SIGNAL( 'clicked()' ), self, SLOT( 'accept()' ) )
			connect( @cancel_button, SIGNAL( 'clicked()' ), self, SLOT( 'reject()' ) )
			if ( mode == 'left' )
				hbox = Qt::HBoxLayout.new()
				hbox.spacing = 4
 				hbox.setDirection( Qt::BoxLayout::LeftToRight )
				hbox.addWidget( @accept_button )
				hbox.addWidget( @cancel_button )
				hbox.addStretch()
			elsif ( mode == 'split' )
				hbox = Qt::HBoxLayout.new()
				hbox.spacing = 4
				hbox.addWidget( @accept_button, 1 )
				hbox.addWidget( @cancel_button, 1 )
			else
				hbox = Qt::HBoxLayout.new()
				hbox.spacing = 4
				hbox.addStretch()
				hbox.addWidget( @accept_button )
				hbox.addWidget( @cancel_button )
			end
			return hbox
		end
		protected :create_action_buttons

		def accepted()
			return result() == Qt::Dialog.Accepted
		end
	end

	class StringListModel < Qt::StringListModel

		def initialize( name )
			super()
			@name = name
		end

		def flags( index )
			return Qt::ItemIsSelectable | Qt::ItemIsEnabled
		end

		def headerData( section, orientation, role = Qt::DisplayRole )
			return Qt::Variant.new() if ( role != Qt::DisplayRole )
			return Qt::Variant.new() if ( orientation != Qt::Horizontal )
			return Qt::Variant.new( @name )
		end

	end

	class PluginsManagerConfigDialog < BaseDialog

		slots 'move_up()', 'move_down()', 'add_plugin()', 'remove_plugin()'

		def initialize( values )
			super( values )

			setWindowTitle( I18N.get( 'gui.pluginsmanager.config.title', @values['script_name'] ) )

			layout = Qt::VBoxLayout.new( self )
			layout.margin = 4
			layout.spacing =4

			group = Qt::GroupBox.new( I18N.get( 'gui.pluginsmanager.config.sites' ), self )
			group_layout = Qt::GridLayout.new( group )
			group_layout.margin = 4
			group_layout.spacing = 4
			group.setLayout( group_layout )
			group_layout.setColumnStretch( 1, 1 )
			group_layout.setColumnStretch( 3, 1 )
			group_layout.setRowStretch( 0, 1 )
			group_layout.setRowStretch( 3, 1 )

			@move_up_button = Qt::PushButton.new( I18N.get( 'gui.pluginsmanager.config.moveup' ), group )
			group_layout.addWidget( @move_up_button, 1, 0 )
			@move_down_button = Qt::PushButton.new( I18N.get( 'gui.pluginsmanager.config.movedown' ), group )
			group_layout.addWidget( @move_down_button, 2, 0 )

			@used_plugins_model = StringListModel.new( I18N.get( 'gui.pluginsmanager.config.inuse' ) )
			@used_plugins_model.setStringList( @values['used_plugins'] )
			@used_plugins_view = Qt::TreeView.new( group )
			@used_plugins_view.rootIsDecorated = false
			@used_plugins_view.sortingEnabled = false
			@used_plugins_view.allColumnsShowFocus = true
			@used_plugins_view.setModel( @used_plugins_model )
			group_layout.addWidget( @used_plugins_view, 0, 1, 4, 1 )

			@add_button = Qt::PushButton.new( I18N.get( 'gui.pluginsmanager.config.add' ), group )
			group_layout.addWidget( @add_button, 1, 2 )
			@remove_button = Qt::PushButton.new( I18N.get( 'gui.pluginsmanager.config.remove' ), group )
			group_layout.addWidget( @remove_button, 2, 2 )

			@unused_plugins_model = StringListModel.new( I18N.get( 'gui.pluginsmanager.config.available' ) )
			@unused_plugins_model.setStringList( @values['unused_plugins'] )
			@unused_plugins_view = Qt::TreeView.new( group )
			@unused_plugins_view.rootIsDecorated = false
			@unused_plugins_view.sortingEnabled = false
			@unused_plugins_view.allColumnsShowFocus = true
			@unused_plugins_view.setModel( @unused_plugins_model )
			group_layout.addWidget( @unused_plugins_view, 0, 3, 4, 1 )

			layout.addWidget( group )

			group = Qt::GroupBox.new( I18N.get( 'gui.pluginsmanager.config.misc' ), self )
			group_layout = Qt::VBoxLayout.new()
			group_layout.margin = 5
			group_layout.spacing = 4
			group.setLayout( group_layout );

			@cleanup_lyrics_checkbox = Qt::CheckBox.new( group )
			@cleanup_lyrics_checkbox.setChecked( values['cleanup_lyrics'].to_s() == 'true' )
			@cleanup_lyrics_checkbox.setText( I18N.get( 'gui.pluginsmanager.config.cleanup' ) )
			group_layout.addWidget( @cleanup_lyrics_checkbox );

			@write_log_checkbox = Qt::CheckBox.new( group )
			@write_log_checkbox.setChecked( values['write_log'].to_s() == 'true' )
			@write_log_checkbox.setText( I18N.get( 'gui.pluginsmanager.config.writelog', '$HOME/.wikilyrics.log' ) )
			group_layout.addWidget( @write_log_checkbox );

			layout.addWidget( group )

			buttons = create_action_buttons()
			layout.addLayout( buttons )

			connect( @move_up_button, SIGNAL('clicked()'), self, SLOT('move_up()') )
			connect( @move_down_button, SIGNAL('clicked()'), self, SLOT('move_down()') )
			connect( @add_button, SIGNAL('clicked()'), self, SLOT('add_plugin()') )
			connect( @remove_button, SIGNAL('clicked()'), self, SLOT('remove_plugin()') )

			update_accept_button()

			resize( 400, 280 )
		end

		def insert_item( view, pos, value, select )
			model = view.model()
			pos = model.rowCount() if ( pos < 0 )
			model.insertRows( pos, 1 )
			idx = model.index( pos )
			model.setData( idx, value )
			if ( select )
				sel_model = view.selectionModel()
				sel_model.clear()
				sel_model.setCurrentIndex( idx, Qt::ItemSelectionModel::Select )
			end
		end

		def remove_item( view, pos )
			model = view.model()
			idx = model.index( pos )
			value = idx.data( Qt::DisplayRole )
			model.removeRows( idx.row(), 1 )
			return value
		end

		def move_up()
			cur_idx = @used_plugins_view.currentIndex()
			return if ( ! cur_idx.isValid() )
			cur_pos = cur_idx.row()
			return if ( cur_pos == 0 )
			plugin = remove_item( @used_plugins_view, cur_pos )
			insert_item( @used_plugins_view, cur_pos-1, plugin, true )
		end

		def move_down()
			cur_idx = @used_plugins_view.currentIndex()
			return if ( ! cur_idx.isValid() )
			cur_pos = cur_idx.row()
			return if ( cur_pos == @used_plugins_view.model().rowCount()-1 )
			plugin = remove_item( @used_plugins_view, cur_pos )
			insert_item( @used_plugins_view, cur_pos+1, plugin, true )
		end

		def add_plugin()
			cur_idx = @unused_plugins_view.currentIndex()
			return if ( ! cur_idx.isValid() )
			plugin = remove_item( @unused_plugins_view, cur_idx.row() )
			insert_item( @used_plugins_view, -1, plugin, false )
			update_accept_button()
		end

		def remove_plugin()
			cur_idx = @used_plugins_view.currentIndex()
			return if ( ! cur_idx.isValid() )
			plugin = remove_item( @used_plugins_view, cur_idx.row() )
			insert_item( @unused_plugins_view, -1, plugin, false )
			update_accept_button()
		end

		def update_accept_button()
			@accept_button.setEnabled( @used_plugins_model.rowCount() > 0 )
		end

		def accept()
			@values = {
				'cleanup_lyrics' => @cleanup_lyrics_checkbox.isChecked(),
				'write_log' => @write_log_checkbox.isChecked(),
				'used_plugins' => @used_plugins_model.stringList(),
				'script_name' => @values['script_name'],
				'unused_plugins' => @unused_plugins_model.stringList()
			}
			super()
		end

	end


	class WikiPluginConfigDialog < BaseDialog

		slots 'toggle_submit_checked( bool )'
		slots 'toggle_review_checked( bool )'

		def initialize( values )
			super( values )

			setWindowTitle( I18N.get( 'gui.wikiplugin.config.title', values['site_name'] ) )

			layout = Qt::GridLayout.new( self )
			layout.margin = 4
			layout.spacing = 4
			layout.setRowStretch( 2, 1 )

			group = Qt::GroupBox.new( I18N.get( 'gui.wikiplugin.config.general' ), self )
			group_layout = Qt::GridLayout.new()
			group_layout.margin = 5
			group_layout.spacing = 4
			group_layout.setRowStretch( 4, 1 )
			group.setLayout( group_layout )

			@submit_checkbox = Qt::CheckBox.new( group )
			@submit_checkbox.setChecked( values['submit'].to_s() == 'true' )
			@submit_checkbox.setText( I18N.get( 'gui.wikiplugin.config.submitcontents', values['site_name'] ) )
			group_layout.addWidget( @submit_checkbox, 0, 0 )

			@review_checkbox = Qt::CheckBox.new( group )
			@review_checkbox.setEnabled( @submit_checkbox.isChecked() )
			@review_checkbox.setChecked( @review_checkbox.isEnabled() && values['review'].to_s() == 'true' )
			@review_checkbox.setText( I18N.get( 'gui.wikiplugin.config.promptforreview' ) )
			group_layout.addWidget( @review_checkbox, 1, 0 )

			@prompt_autogen_checkbox = Qt::CheckBox.new( group )
			@prompt_autogen_checkbox.setEnabled( @review_checkbox.isChecked() )
			@prompt_autogen_checkbox.setChecked( @prompt_autogen_checkbox.isEnabled() && values['prompt_autogen'].to_s() == 'true' )
			@prompt_autogen_checkbox.setText( I18N.get( 'gui.wikiplugin.config.editautogenerated' ) )
			group_layout.addWidget( @prompt_autogen_checkbox, 2, 0 )

			@prompt_new_checkbox = Qt::CheckBox.new( group )
			@prompt_new_checkbox.setEnabled( @review_checkbox.isChecked() )
			@prompt_new_checkbox.setChecked( @prompt_new_checkbox.isEnabled() && values['prompt_new'].to_s() == 'true' )
			@prompt_new_checkbox.setText( I18N.get( 'gui.wikiplugin.config.promptonnolyrics' ) )
			group_layout.addWidget( @prompt_new_checkbox, 3, 0 )

			layout.addWidget( group, 0, 0 )

			group = Qt::GroupBox.new( I18N.get( 'gui.wikiplugin.config.login' ), self )
			group_layout = Qt::GridLayout.new()
			group_layout.setRowStretch( 2, 1 )
			group_layout.margin = 5
			group_layout.spacing = 4
			group.setLayout( group_layout )

			label = Qt::Label.new( I18N.get( 'gui.wikiplugin.config.username' ), group )
			group_layout.addWidget( label, 0, 0 );

			@username_lineedit = Qt::LineEdit.new( values['username'], group )
			group_layout.addWidget( @username_lineedit, 0, 1 );

			label = Qt::Label.new( I18N.get( 'gui.wikiplugin.config.password' ), group )
			group_layout.addWidget( label, 1, 0 )

			@password_lineedit = Qt::LineEdit.new( values['password'], group )
			@password_lineedit.setEchoMode( Qt::LineEdit::Password )
			group_layout.addWidget( @password_lineedit, 1, 1 )

			layout.addWidget( group, 1, 0 )

			buttons = create_action_buttons()
			layout.addLayout( buttons, 2, 0 )

			connect( @submit_checkbox, SIGNAL('toggled(bool)'), self, SLOT('toggle_submit_checked(bool)') )
			connect( @review_checkbox, SIGNAL('toggled(bool)'), self, SLOT('toggle_review_checked(bool)') )

			resize( 300, 50 )
		end

		def toggle_submit_checked( checked )
			@review_checkbox.setEnabled( checked )
			if ( ! checked )
				@review_checkbox.setChecked( false )
				@prompt_autogen_checkbox.setChecked( false )
				@prompt_new_checkbox.setChecked( false )
			end
		end

		def toggle_review_checked( checked )
			@prompt_autogen_checkbox.setEnabled( checked )
			@prompt_new_checkbox.setEnabled( checked )
			if ( ! checked )
				@prompt_autogen_checkbox.setChecked( false )
				@prompt_new_checkbox.setChecked( false )
			end
		end

		def accept()
			@values = {
				'submit'			=> @submit_checkbox.isChecked(),
				'review'			=> @review_checkbox.isChecked(),
				'prompt_autogen'	=> @prompt_autogen_checkbox.isChecked(),
				'prompt_new'		=> @prompt_new_checkbox.isChecked(),
				'username'			=> @username_lineedit.text(),
				'password'			=> @password_lineedit.text(),
			}
			super()
		end

	end

	class WikiSubmitSongDialog < BaseDialog

		slots 'toggle_instrumental_checked( bool )'

		def initialize( values )
			super( values )

			edit_mode = @values['edit_mode'].to_s() == 'true'
			setWindowTitle( I18N.get( 'gui.wiki.submitsong.title.' + (edit_mode ? 'edit' : 'submit'), values['site_name'] ) )

			layout = Qt::GridLayout.new( self )
			layout.margin = 4
			layout.spacing = 4

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.url' )}</b>", self ), 0, 0, Qt::AlignRight )
			@url_lineedit = Qt::LineEdit.new( values['url'], self )
			@url_lineedit.setEnabled( !edit_mode )
			layout.addWidget( @url_lineedit, 0, 1, 1, 3 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.artist' )}</b>", self ), 1, 0, Qt::AlignRight )
			@artist_lineedit = Qt::LineEdit.new( values['artist'], self )
			layout.addWidget( @artist_lineedit, 1, 1 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.song' )}</b>", self ), 1, 2, Qt::AlignRight )
			@title_lineedit = Qt::LineEdit.new( values['title'], self )
			layout.addWidget( @title_lineedit, 1, 3 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.credits' )}</b>", self ), 2, 0, Qt::AlignRight )
			@credits_lineedit = Qt::LineEdit.new( values['credits'], self )
			layout.addWidget( @credits_lineedit, 2, 1 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.lyricist' )}</b>", self ), 2, 2, Qt::AlignRight )
			@lyricist_lineedit = Qt::LineEdit.new( values['lyricist'], self )
			layout.addWidget( @lyricist_lineedit, 2, 3 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.year' )}</b>", self ), 3, 0, Qt::AlignRight )
			@year_spinbox = Qt::SpinBox.new( self )
			@year_spinbox.setRange( 1900, Date.today().year )
			@year_spinbox.setValue( values['year'] )
			layout.addWidget( @year_spinbox, 3, 1 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.album' )}</b>", self ), 3, 2, Qt::AlignRight )
			@album_lineedit = Qt::LineEdit.new( values['album'], self )
			layout.addWidget( @album_lineedit, 3, 3 )

			@instrumental_checkbox = Qt::CheckBox.new( self )
			@instrumental_checkbox.setChecked( @values['instrumental'].to_s() == 'true' )
			@instrumental_checkbox.setText( I18N.get( 'gui.wiki.submitsong.instrumental' ) )
			layout.addWidget( @instrumental_checkbox, 4, 0, 1, 3 )

			@lyrics_text = Qt::TextEdit.new( self )
			@lyrics_text.setLineWrapMode( Qt::TextEdit::NoWrap )
			@lyrics_text.setPlainText( values['lyrics'] )
			@lyrics_text.setDisabled( @instrumental_checkbox.isChecked() )
			layout.addWidget( @lyrics_text, 5, 0, 1, 4 )

			@autogen_checkbox = Qt::CheckBox.new( self )
			@autogen_checkbox.setChecked( true )
			@autogen_checkbox.setText( I18N.get( 'gui.common.autogenerated' ) )
			layout.addWidget( @autogen_checkbox, 6, 0, 1, 4 )

			buttons = create_action_buttons( 'split', I18N.get( 'gui.common.submit' ) )
			layout.addLayout( buttons, 7, 0, 1, 4 )

			resize( 600, 400 )

			connect( @instrumental_checkbox, SIGNAL('toggled(bool)'), self, SLOT('toggle_instrumental_checked(bool)') )

		end

		def toggle_instrumental_checked( checked )
			@lyrics_text.setDisabled( checked )
		end

		def accept()
			@values = {
				'url'			=> @url_lineedit.text(),
				'artist'		=> @artist_lineedit.text(),
				'year'			=> @year_spinbox.value(),
				'album'			=> @album_lineedit.text(),
				'title'			=> @title_lineedit.text(),
				'lyrics'		=> @lyrics_text.toPlainText(),
				'instrumental'	=> @instrumental_checkbox.isChecked(),
				'lyricist'		=> @lyricist_lineedit.text(),
				'credits'		=> @credits_lineedit.text(),
				'autogen'		=> @autogen_checkbox.isChecked()
			}
			super()
		end

	end

	class WikiSubmitAlbumDialog < BaseDialog

		slots 'browse_image()'

		def initialize( values )
			super( values )

			@values = values

			setWindowTitle( I18N.get( 'gui.wiki.submitalbum.title', values['site_name'] ) )

			layout = Qt::GridLayout.new( self )
			layout.margin = 4
			layout.spacing = 4

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.url' )}</b>", self ), 0, 0, Qt::AlignRight )
			@url_lineedit = Qt::LineEdit.new( values['url'], self )
			layout.addWidget( @url_lineedit, 0, 1, 1, 3 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.artist' )}</b>", self ), 1, 0, Qt::AlignRight )
			@artist_lineedit = Qt::LineEdit.new( values['artist'], self )
			layout.addWidget( @artist_lineedit, 1, 1, 1, 3 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.released' )}</b>", self ), 2, 0, Qt::AlignRight )
			@released_lineedit = Qt::LineEdit.new( values['released'], self )
			layout.addWidget( @released_lineedit, 2, 1 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.album' )}</b>", self ), 2, 2, Qt::AlignRight )
			@album_lineedit = Qt::LineEdit.new( values['album'], self )
			layout.addWidget( @album_lineedit, 2, 3 )

			layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.image' )}</b>", self ), 3, 0, Qt::AlignRight )
			img_layout = Qt::HBoxLayout.new()
			@image_path_lineedit = Qt::LineEdit.new( values['image_path'], self )
			if ( ! values.include?( 'image_path' ) )
				@image_path_lineedit.setEnabled( false )
				@image_path_lineedit.setText( I18N.get( 'gui.wiki.submitalbum.coverfound' ) )
			else
				@image_button = Qt::PushButton.new( '...', self )
				@image_button.setMaximumWidth( 40 )
				connect( @image_button, SIGNAL( 'clicked()' ), self, SLOT( 'browse_image()' ) )
			end
			img_layout.addWidget( @image_path_lineedit )
			img_layout.addWidget( @image_button ) if ( @values.include?( 'image_path' ) )
			layout.addLayout( img_layout, 3, 1, 1, 3 )

			@tracks_text = Qt::TextEdit.new( self )
			@tracks_text.setLineWrapMode( Qt::TextEdit::NoWrap )
			@tracks_text.setPlainText( values['tracks'] )
			layout.addWidget( @tracks_text, 4, 0, 1, 4 )

			@autogen_checkbox = Qt::CheckBox.new( self )
			@autogen_checkbox.setChecked( true )
			@autogen_checkbox.setText( I18N.get( 'gui.common.autogenerated' ) )
			layout.addWidget( @autogen_checkbox, 5, 0, 1, 4 )

			buttons = create_action_buttons( 'split', I18N.get( 'gui.common.submit' ) )
			layout.addLayout( buttons, 6, 0, 1, 4 )

			resize( 600, 400 )
		end

		def browse_image()
			dirname = @image_path_lineedit.text().strip()
			dirname = QT4.empty_string?( dirname ) ? (ENV['HOME'] ? ENV['HOME'] : '.') : File.dirname( dirname )
			dialog = Qt::FileDialog.new()
			dialog.setWindowTitle( I18N.get( 'gui.wiki.uploadcover.browseimage.title' ) )
			dialog.setFilters( [
				I18N.get( 'gui.wiki.uploadcover.browseimage.images' ) + ' (*.jpg *.JPG *.png *.PNG *.bmp *.BMP)',
				I18N.get( 'gui.wiki.uploadcover.browseimage.allfiles' ) + ' (*)'
			] )
			dialog.setDirectory( dirname )
			dialog.setFileMode( Qt::FileDialog::ExistingFile )
			 if ( dialog.exec() == Qt::Dialog::Accepted )
        		image_path = dialog.selectedFiles()[0]
				@image_path_lineedit.setText( image_path.strip() ) if ( ! QT4.empty_string?( image_path ) )
			end
		end

		def accept()
			aux = {
				'url'		=> @url_lineedit.text(),
				'artist'	=> @artist_lineedit.text(),
				'released'	=> @released_lineedit.text(),
				'album'		=> @album_lineedit.text(),
				'tracks'	=> @tracks_text.toPlainText(),
				'autogen'	=> @autogen_checkbox.isChecked()
			}
			aux['image_path'] = @image_path_lineedit.text() if ( @values.include?( 'image_path' ) )
			@values = aux
			super()
		end

	end


	class WikiUploadCoverDialog < BaseDialog

		slots 'browse_image()'

		def initialize( values )
			super( values )

			setWindowTitle( I18N.get( 'gui.wiki.uploadcover.title', values['site_name'] ) )

			layout = Qt::VBoxLayout.new( self )
			layout.margin = 4
			layout.spacing = 4

			group = Qt::GroupBox.new( I18N.get( 'gui.common.album' ), self )
			group_layout = Qt::GridLayout.new()
			group_layout.margin = 5
			group_layout.spacing = 4
			group_layout.setRowStretch( 3, 1 )
			group.setLayout( group_layout )

			label = Qt::Label.new( I18N.get( 'gui.common.artist' ), group )
			group_layout.addWidget( label, 0, 0 )
			@artist_lineedit = Qt::LineEdit.new( values['artist'], group )
			group_layout.addWidget( @artist_lineedit, 0, 1 )

			label = Qt::Label.new( I18N.get( 'gui.common.album' ), group )
			group_layout.addWidget( label, 1, 0 )
			@album_lineedit = Qt::LineEdit.new( values['album'], group )
			group_layout.addWidget( @album_lineedit, 1, 1 )

			label = Qt::Label.new( I18N.get( 'gui.common.year' ), group )
			group_layout.addWidget( label, 2, 0 )
			@year_spinbox = Qt::SpinBox.new( group )
			@year_spinbox.setRange( 1900, Date.today().year() )
			@year_spinbox.setValue( values['year'] )
			group_layout.addWidget( @year_spinbox, 2, 1 )

			layout.addWidget( group )

			group = Qt::GroupBox.new( I18N.get( 'gui.common.image' ), self )
			group_layout = Qt::GridLayout.new()
			group_layout.margin = 5
			group_layout.spacing = 4
			group_layout.setRowStretch( 1, 1 )
			group.setLayout( group_layout )

			label = Qt::Label.new( I18N.get( 'gui.wiki.uploadcover.imagepath' ), group )
			group_layout.addWidget( label, 0, 0 )
			@image_path_lineedit = Qt::LineEdit.new( values['image_path'], group )
			@image_path_lineedit.setText( values['image_path'] )
			group_layout.addWidget( @image_path_lineedit, 0, 1 )

			@image_button = Qt::PushButton.new( '...', group )
			@image_button.setMaximumWidth( 40 )
			group_layout.addWidget( @image_button, 0, 2 )

			connect( @image_button, SIGNAL( 'clicked()' ), self, SLOT( 'browse_image()' ) )

			layout.addWidget( group )

			buttons = create_action_buttons()
			layout.addLayout( buttons )

			resize( 350, 100 )
		end

		def browse_image()
			dirname = @image_path_lineedit.text().strip()
			dirname = QT4.empty_string?( dirname ) ? (ENV['HOME'] ? ENV['HOME'] : '.') : File.dirname( dirname )
			dialog = Qt::FileDialog.new()
			dialog.setWindowTitle( I18N.get( 'gui.wiki.uploadcover.browseimage.title' ) )
			dialog.setFilters( [
				I18N.get( 'gui.wiki.uploadcover.browseimage.images' ) + ' (*.jpg *.JPG *.png *.PNG *.bmp *.BMP)',
				I18N.get( 'gui.wiki.uploadcover.browseimage.allfiles' ) + ' (*)'
			] )
			dialog.setDirectory( dirname )
			dialog.setFileMode( Qt::FileDialog::ExistingFile )
			 if ( dialog.exec() == Qt::Dialog::Accepted )
        		image_path = dialog.selectedFiles()[0]
				@image_path_lineedit.setText( image_path.strip() ) if ( ! QT4.empty_string?( image_path ) )
			end
		end

		def accept()
			@values = {
				'artist'		=> @artist_lineedit.text(),
				'album'			=> @album_lineedit.text(),
				'year'			=> @year_spinbox.value(),
				'image_path'	=> @image_path_lineedit.text()
			}
			super()
		end

	end


	class WikiFixPagesDialog < BaseDialog

		slots 'fetch_page()'
		slots 'fix_page()'
		slots 'submit_page()'

		def initialize( values )
			super( values )

			setWindowTitle( "#{values['site_name']} - #{values['title']}" )

			layout = Qt::GridLayout.new( self )
			layout.margin = 4
			layout.spacing = 4

			@url_lineedit = Qt::LineEdit.new( values['url'], self )
			url_layout = Qt::HBoxLayout.new()
			url_layout.spacing = 4
			url_layout.setDirection( Qt::BoxLayout::LeftToRight )
			url_layout.addWidget( Qt::Label.new( "<b>#{I18N.get( 'gui.common.url' )}</b>", self ) )
			url_layout.addWidget( @url_lineedit )
			layout.addLayout( url_layout, 0, 0 )

			@page_text = Qt::TextEdit.new( self )
			@page_text.setLineWrapMode( Qt::TextEdit::NoWrap )
			layout.addWidget( @page_text, 1, 0 )

			@fetch_button = Qt::PushButton.new( I18N.get( 'gui.common.load' ), self )
			@fix_button = Qt::PushButton.new( I18N.get( 'gui.common.fix' ), self )
			@submit_button = Qt::PushButton.new( I18N.get( 'gui.common.submit' ), self )
			buttons_layout = Qt::HBoxLayout.new()
			buttons_layout.spacing = 4
			buttons_layout.addStretch()
			buttons_layout.addWidget( @fetch_button )
			buttons_layout.addWidget( @fix_button )
			buttons_layout.addWidget( @submit_button )
			layout.addLayout( buttons_layout, 2, 0 )

			connect( @fetch_button, SIGNAL( 'clicked()' ), self, SLOT( 'fetch_page()' ) )
			connect( @fix_button, SIGNAL( 'clicked()' ), self, SLOT( 'fix_page()' ) )
			connect( @submit_button, SIGNAL( 'clicked()' ), self, SLOT( 'submit_page()' ) )

			resize( 600, 400 )
		end

		def fetch_page()
			if ( page_content = @values['fetch_func'].call( @url_lineedit.text() ) )
				@original_content = page_content
				@original_url = @url_lineedit.text()
				@page_text.setPlainText( page_content )
			end
		end

		def fix_page()
			@page_text.setPlainText( @values['fix_func'].call( @page_text.toPlainText() ) ) if ( @original_content )
		end

		def submit_page()

			if ( @original_content && @original_content != (page_content = @page_text.toPlainText()) )
				if ( @values['submit_func'].call( @original_url, page_content ) )
					@original_content = page_content
					puts I18N.get( 'gui.wiki.fixpages.success', @original_url )
				else
					puts I18N.get( 'gui.wiki.fixpages.error', @original_url )
				end
			end
		end

	end


	class SearchLyricsDialog < BaseDialog

		def initialize( values )
			super( values )

			setWindowTitle( I18N.get( 'gui.searchlyrics.title' ) )

			layout = Qt::VBoxLayout.new( self )
			layout.margin = 4
			layout.spacing = 4

			group = Qt::GroupBox.new( I18N.get( 'gui.searchlyrics.search' ), self )
			group_layout = Qt::GridLayout.new()
			group_layout.margin = 5
			group_layout.spacing = 4
			group_layout.setRowStretch( 4 , 1 )
			group.setLayout( group_layout )

			label = Qt::Label.new( I18N.get( 'gui.common.artist' ), group )
			group_layout.addWidget( label, 0, 0 )
			@artist_lineedit = Qt::LineEdit.new( values['artist'], group )
			group_layout.addWidget( @artist_lineedit, 0, 1 )

			label = Qt::Label.new( I18N.get( 'gui.common.song' ), group )
			group_layout.addWidget( label, 1, 0 )
			@title_lineedit = Qt::LineEdit.new( values['title'], group )
			group_layout.addWidget( @title_lineedit, 1, 1 )

			label = Qt::Label.new( I18N.get( 'gui.common.album' ), group )
			group_layout.addWidget( label, 2, 0 )
			@album_lineedit = Qt::LineEdit.new( values['album'], group )
			group_layout.addWidget( @album_lineedit, 2, 1 )

			label = Qt::Label.new( I18N.get( 'gui.common.year' ), group )
			group_layout.addWidget( label, 3, 0 )
			@year_spinbox = Qt::SpinBox.new( group )
			@year_spinbox.setRange( 1900, Date.today().year() )
			@year_spinbox.setValue( values['year'] )
			group_layout.addWidget( @year_spinbox, 3, 1 )

			layout.addWidget( group )

			buttons = create_action_buttons()
			layout.addLayout( buttons )

			resize( 300, 100 )
		end

		def accept()
			@values = {
				'artist' => @artist_lineedit.text(),
				'title'  => @title_lineedit.text(),
				'album'  => @album_lineedit.text(),
				'year'   => @year_spinbox.value(),
			}
			super()
		end

	end


	class LyricsDialog < BaseDialog

		def initialize( values )
			super( values )

			title = I18N.get( 'gui.lyrics.title', @values['title'], @values['artist'] )
			title += " [#{@values['site_name']}]" if @values['site_name']
			setWindowTitle( title  )

			layout = Qt::GridLayout.new( self )
			layout.margin = 4
			layout.spacing = 4

			@lyrics_text = Qt::TextEdit.new( self )
			@lyrics_text.setPlainText( values['lyrics'] )
			@lyrics_text.setLineWrapMode( Qt::TextEdit::NoWrap )
			layout.addWidget( @lyrics_text, 0, 0 )

			resize( 350, 400 )
		end

	end


end
