#  Copyright (C) 2006-2007 by Sergio Pistone
#  sergio_pistone@yahoo.com.ar
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))
require 'utils/http'
require 'utils/xmlhash'
require 'utils/formdata'
require 'utils/strings'
require 'utils/htmlentities'
require 'i18n/i18n'
require 'gui/gui'
require 'lyrics'

require 'md5'
require 'cgi'

class WikiLyrics < Lyrics

	@@NAME    = 'WL'
	@@VERSION = '0.12.4'

	@@FOLLOW_REDIRECTS = 3

	def WikiLyrics.version()
		return String.new( @@VERSION )
	end

	attr_reader :review, :username, :password
	attr_writer :review

	def initialize( cleanup_lyrics=true, log_file=$DFLT_LOG_FILE, review=true, username=nil, password=nil )
		super( cleanup_lyrics, log_file )
		@logged_in = false
		@review = review
		@username = username
		@password = password
	end

	def logged_in?()
		return @logged_in
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		return { 'url' => build_song_url( artist, title ) }
	end

	def page_error?( page_body )

		# database error
		db_error = /<!-- start content -->\s*(<p>|)A database query syntax error has occurred\.\s*This may indicate a bug in the software\.*/m.match( page_body )

 		# page title error
		title_error = /<!-- start content -->\s*(<p>|)The requested page title was invalid, empty, or an incorrectly linked inter-language or inter-wiki title\. It may contain one( or|) more characters which cannot be used in titles\.*/m.match( page_body )

		return db_error != nil || title_error != nil

	end
	protected :page_error?

	def fetch_lyrics_page( url, post )
		return nil, nil if ( url == nil )
		page_body, page_url = super( "#{url}&action=raw&ctype=text/javascript", post )
		page_url.sub!( /&action=raw&ctype=text\/javascript$/, '' )
		return nil, page_url if ( page_body == nil )

		# work around redirects pages
		count = 0
		article = parse_url( page_url )
		while ( count < @@FOLLOW_REDIRECTS && (md = /#redirect \[\[([^\]]+)\]\]/i.match( page_body )) )
			target_article = cleanup_article( md[1] )
			if ( article == target_article )
				log( "ERROR (circular redirect found)" ) if ( log?() )
				return nil, page_url
			end
			page_url = "http://#{site_host()}/index.php?title=#{md[1].gsub(' ', '_')}"
			log( "Found redirect page (#{count+1}), following link to #{page_url}... ", 0 ) if ( log?() )
			response, page_url = HTTP.fetch_page_get( "#{page_url}&action=raw&ctype=text/javascript" )
			page_url.sub!( /&action=raw&ctype=text\/javascript$/, '' )
			if ( log?() )
				if ( response && response.body() )
					log( 'OK' )
					log( response.body(), 2 ) if ( long_log?() )
				else
					log( 'ERROR' )
				end
			end
			page_body = response ? response.body() : nil
			return nil, page_url if ( page_body == nil )
			count = count + 1
			article = target_article
		end
		return page_body, page_url
	end

	def check_lyrics_page( page_body, artist, title, album=nil, year=nil )
		return ! page_error?( page_body )
	end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { 'url' => build_song_search_url( artist, title ) }
	end

	def lyrics_from_url( url, artist, title, album=nil, year=nil )
		# we fetch wiki lyrics in raw mode so we need this specific format
		if ( ! url.index( 'index.php?title=' ) )
			if ( ! Strings.empty?( artist ) && ! Strings.empty?( title ) )
				url = build_song_url( artist, title )
			else
				a, t = self.class.parse_song_url( title )
				url = build_song_url( a, t )
			end
		end
		return super( url, artist, title, album, year )
	end

	def parse_suggestions( url, body, artist, title, album=nil, year=nil )
		suggestions = parse_search_results( body, true )
		idx = 0
		suggestions.size.times do
			sugg = suggestions[idx]
			sugg['artist'], sugg['title'] = self.class.parse_song_url( sugg['title'] )
			if ( Strings.empty?( sugg['artist'] ) || Strings.empty?( sugg['title'] ) || /\ \([0-9]{4,4}\)$/.match( sugg['title'] ) )
				suggestions.delete_at( idx )
			else
				idx += 1
			end
		end
		return suggestions
	end

	def login( username=@username, password=@password, force=false )

		return true if ( ! force && @logged_in && username == @username && password == @password )

		@username, @password = username, password
		notify = ! @logged_in || ! force

		if ( ! @username || ! @password )
			notify( I18N.get( 'wiki.login.error', @username ? @username : '<NONE>' ) ) if ( notify )
			return @logged_in = false
		end

		notify( I18N.get( 'wiki.login.attempt', @username ) ) if ( notify )

		headers = { 'Keep-Alive'=>'300', 'Connection'=>'keep-alive' }
		resp, url = HTTP.fetch_page_get( "http://#{site_host()}/index.php?title=Special:Userlogin", headers )
		@cookie = resp.response['set-cookie'].split( '; ' )[0]

		params = { 'wpName'=>@username, 'wpPassword'=>@password, 'wpLoginattempt'=>'Log In' }
		headers.update( { 'Cookie'=>@cookie } )
		resp, url = HTTP.fetch_page_post( "http://#{site_host()}/index.php?title=Special:Userlogin&action=submitlogin", params, headers, -1 )

		# Read more cookies info
		resp.each do |key, val|
			if ( key == 'set-cookie' )
				val.split( /[;,] /).each do |c_entry|
					if ( c_entry.split( '=' )[0] == 'lyrikiUserID' || c_entry.split( '=' )[0] == 'lyrikiUserName' )
						@cookie += "; #{c_entry}"
					end
				end
			end
		end

		data = resp.body()
		data.gsub!( /[ \t\n]+/, ' ' )

		@logged_in = (/<h2>Login error:<\/h2>/.match( data ) == nil)
		@logged_in = (/<h1 class="firstHeading">Login successful<\/h1>/.match( data ) != nil) if ( @logged_in ) # recheck

		notify( I18N.get( 'wiki.login.' + (@logged_in ? 'success' : 'error'), @username ) ) if ( notify )

		return @logged_in

	end

	def restore_session_params( cookie, user_md5, pass_md5, username=@username, password=@password )

		if ( cookie != nil && username != nil && user_md5 != nil && pass_md5 != nil &&
			 MD5.hexdigest( username ) == user_md5 &&
			 MD5.hexdigest( password.to_s() ) == pass_md5.to_s() )
			@username = username
			@password = password
			@cookie = cookie
			@logged_in = true
			notify( I18N.get( 'wiki.session.restore.success', @username ) )
			return true
		else
			notify( I18N.get( 'wiki.session.restore.error', @username ) )
			return false
		end

	end

	def restore_session( session_file, username=@username, password=@password )
		values = { 'cookie' => nil, 'user_md5' => nil, 'pass_md5' => nil }
		if ( XMLHash.read( session_file, values ) )
			return restore_session_params( values['cookie'], values['user_md5'], values['pass_md5'], username, password )
		else
			notify( I18N.get( 'wiki.session.restore.notfound' ) )
			return false
		end
	end

	def save_session( session_file )
		if ( ! @logged_in )
			notify( I18N.get( 'wiki.session.save.error.notloggedin' ) )
			return false
		end
		values = { 'cookie' => @cookie, 'user_md5' => MD5.hexdigest( @username ), 'pass_md5' => MD5.hexdigest( @password ) }
		if ( XMLHash.write( session_file, values ) )
			notify( I18N.get( 'wiki.session.save.success', @username ) )
			return true
		else
			notify( I18N.get( 'wiki.session.save.error', @username ) )
			return false
		end
	end


	def fetch_page_edit_params( url )
		headers = { 'Keep-Alive'=>'300', 'Connection'=>'keep-alive', 'Referer'=>"#{url}", 'Cookie'=>@cookie }
		resp, url = HTTP.fetch_page_get( "#{url}&action=edit", headers )

		edit_params = {}
		return edit_params if ( resp.code != '200' )

		body = resp.body()
		body.tr_s!( " \n\r\t", ' ' )

		if ( md = /<input type=['"]hidden['"] value=['"]([a-fA-F0-9\\]*)['"] name=['"]wpEditToken['"] ?\/>/.match( body ) )
			edit_params['edit_token'] = md[1]
			if ( md = /<input type=['"]hidden['"] value=['"]([0-9]+)['"] name=['"]wpEdittime['"] ?\/>/.match( body ) )
				edit_params['edit_time'] = md[1]
			end
			if ( md = /<input type=['"]hidden['"] value=['"]([0-9]+)['"] name=['"]wpStarttime['"] ?\/>/.match( body ) )
				edit_params['start_time'] = md[1]
			end
			edit_params['logged_in'] = (/<li id="pt-logout">/.match( body ) != nil)
		end

		return edit_params
	end
	protected :fetch_page_edit_params


	def debug( text )
		`kdialog --msgbox #{Strings.shell_quote( text )}`
	end

	def submit_page( url, page_content, summary='', watch=true, retries=1, auto_login=true )

		return false if ( ! logged_in?() && ! auto_login )

		(retries+1).times do

			begin

				next if ( ! logged_in?() && ! login() )

				url = build_url( parse_url( url ) )

				# Try to get the edit token for url, can't continue without it:
				edit_params = fetch_page_edit_params( url )
				next if ( ! edit_params['edit_token'] )

				if ( ! edit_params['logged_in'] ) # site says user is not logged in (session has expired), force relogin

					debug( "NOT LOGGED IN" )

					next if ( ! login( @username, @password, true ) )

					debug( "FORCED LOGIN OK" )

					# after being successfully logged in, refetch the edit page
					edit_params = fetch_page_edit_params( url )
					next if ( ! edit_params['edit_token'] || ! edit_params['logged_in'] )

					debug( "ALL OK" )

				end

				params = [
					MultipartFormData.text_param( 'wpTextbox1', page_content ),
					MultipartFormData.text_param( 'wpSummary', summary ),
					MultipartFormData.text_param( 'wpWatchthis', watch ? 'on' : 'off' ),
					MultipartFormData.text_param( 'wpSave', 'Save page' ),
					MultipartFormData.text_param( 'wpSection', '' ),
					MultipartFormData.text_param( 'wpStarttime', edit_params['start_time'].to_s() ), # the new revision time
					MultipartFormData.text_param( 'wpEdittime', edit_params['edit_time'].to_s() ), # the previous revision time
					MultipartFormData.text_param( 'wpEditToken', edit_params['edit_token'] ),
				]

				headers = {
					'Keep-Alive'  => '300',
					'Connection'  => 'keep-alive',
					'Referer'     => "http://#{site_host()}#{url}&action=edit",
					'Cookie'      => @cookie,
				}

				resp, url = HTTP.fetch_page_post_form_multipart( "#{url}&action=submit", params, headers, -1 )

				return true if ( resp.code == '302' ) # we should have received a redirect code

			rescue TimeoutError
			end
		end

		return false
	end

	def submit_redirect_page( url, link, summary=nil )

		raise ArgumentError if ( Strings.empty?( url ) || Strings.empty?( link ) )

		if ( submit_page( url, "#redirect #{link}", summary ) )
			notify( I18N.get( 'wiki.submitredirect.success', link ) )
			return url
		else
			notify( I18N.get( 'wiki.submitredirect.error', link ) )
			return nil
		end

	end

	def upload_file( src_file, dst_file, mime_type, description='', watch=true, auto_login=true )

		if ( ! logged_in?() )
			return false if ( ! auto_login || ! login() )
		end

		begin
			data = File.new( src_file ).read()
		rescue Exception
			return false
		end

		params = [
			MultipartFormData.file_param( 'wpUploadFile', File.basename( src_file ), mime_type, data ),
			MultipartFormData.text_param( 'wpDestFile', dst_file ),
			MultipartFormData.text_param( 'wpUploadDescription', description ),
			MultipartFormData.text_param( 'wpWatchthis', watch ? 'true' : 'false' ),
			MultipartFormData.text_param( 'wpUpload', 'Upload file' ),
		]

		headers = {
			'Keep-Alive'  => '300',
			'Connection'  => 'keep-alive',
			'Referer'     => "http://#{site_host()}/index.php?title=Special:Upload&wpDestFile=#{CGI.escape(dst_file)}",
			'Cookie'      => @cookie,
		}

		begin
			resp, url = HTTP.fetch_page_post_form_multipart( "http://#{site_host()}/index.php?title=Special:Upload", params, headers, -1 )
			if ( resp.code == '302' ) # we should have received a redirect
				return true
			else # error, posibly an expired session: relogin and try again
				login( @username, @password, true )
				resp, url = HTTP.fetch_page_post_form_multipart( "http://#{site_host()}/index.php?title=Special:Upload", params, headers, -1 )
				return resp.code == '302' # again, we should have received a redirect
			end
		rescue TimeoutError
			return false
		end

	end

	def upload_cover_image( image_path, artist, album, year, watch=true )

		album_art_name = build_album_art_name( artist, album, year )
		album_art_desc = build_album_art_description( artist, album, year )
		image_path, mime_type = prepare_image_file( image_path )

		if ( Strings.empty?( image_path ) || Strings.empty?( mime_type ) )
			notify( I18N.get( 'wiki.uploadcover.error.convert' ) )
		else
			notify( I18N.get( 'wiki.uploadcover.uploading', album, artist ) )
			if ( upload_file( image_path, album_art_name, mime_type, album_art_desc, watch ) )
				notify( I18N.get( 'wiki.uploadcover.success', album, artist ) )
			else
				notify( I18N.get( 'wiki.uploadcover.error', album, artist ) )
			end
		end

	end


	def build_tracks( tracks )
		return self.class.build_tracks( tracks )
	end

	def build_album_page( autogen, artist, album, released, tracks, album_art )
		return self.class.build_album_page( autogen, artist, album, released, tracks, album_art )
	end

	def submit_album_page( album, year, tracks, image_path=nil, month=nil, day=nil )

		raise ArgumentError if ( Strings.empty?( album ) || year.to_i() <= 1900 )

		tracks_data = parse_tracks( tracks )
		artist = cleanup_title_token( tracks_data['artist'] )
		raise ArgumentError if ( Strings.empty?( artist ) )

		url = build_album_url( artist, album, year )
		page_data = {
			'site_name'		=> site_name(),
			'url'			=> url,
			'artist'		=> artist,
			'released'		=> month ? "#{month}#{day ? " #{day}" : day}, #{year}" : "#{year}",
			'album'			=> cleanup_title_token( album ),
			'tracks'		=> build_tracks( tracks ),
			'autogen'		=> true
		}

		page_data['album_art_name'] = find_album_art_name( page_data['artist'], page_data['album'], page_data['year'] )
		if ( page_data['album_art_name'] == nil ) # album art not found, we'll attempt to upload it
			page_data['image_path'] = image_path.to_s()
			attempt_upload = true
		else
			attempt_upload = false
		end

		if ( @review && ! GUI.show_wiki_submit_album_dialog( page_data ) )
			notify( I18N.get( 'wiki.submitalbum.cancelled' ) )
			return nil, nil
		else
			ar, al, yr = parse_album_url( page_data['url'] )
			if ( Strings.empty?( yr ) ||  yr.to_i() <= 1900 )
				notify( I18N.get( 'wiki.submitalbum.invalidurl' ) )
				return nil, nil
			end
			page_data['year'] = yr.to_i()
		end

		page_content = build_album_page(
			page_data['autogen'],
			page_data['artist'],
			page_data['album'],
			page_data['released'],
			page_data['tracks'],
			page_data['album_art_name'] ?
				page_data['album_art_name'] :
				build_album_art_name( page_data['artist'], page_data['album'], page_data['year'] )
		)

		if ( attempt_upload && ! Strings.empty?( page_data['image_path'] ) )
			upload_cover_image( page_data['image_path'], page_data['artist'], page_data['album'], page_data['year'] )
		end

		summary = "#{page_data['autogen'] ? 'autogen. ' : ''}album page (#{@@NAME}v#{@@VERSION})"
		if ( submit_page( page_data['url'], page_content, summary ) )
			notify( I18N.get( 'wiki.submitalbum.success', page_data['album'], page_data['artist'] ) )
			return page_data['url'], page_data
		else
			notify( I18N.get( 'wiki.submitalbum.error', page_data['url'] ) )
			return nil, page_data
		end

	end

	def build_song_page( autogen, artist, title, album, year, credits, lyricist, lyrics )
		self.class.build_song_page( autogen, artist, title, album, year, credits, lyricist, lyrics )
	end

	def submit_song_page( lyrics, artist, title, album, year, credits, lyricist, edit_url=nil )

		raise ArgumentError if ( Strings.empty?( artist ) || Strings.empty?( title ) )

		edit_mode = edit_url != nil
		url = edit_mode ? edit_url : build_song_url( artist, title )

		page_data = {
			'site_name'		=> site_name(),
			'edit_mode'		=> edit_mode,
			'url' 			=> url,
			'artist'		=> cleanup_title_token( artist ),
			'year'			=> year.to_s(),
			'album'			=> cleanup_title_token( album.to_s() ),
			'title'			=> cleanup_title_token( title ),
			'lyrics'		=> Strings.cleanup_lyrics( lyrics.to_s() ),
			'instrumental'	=> Strings.empty?( lyrics ),
			'credits'		=> credits.to_s(),
			'lyricist'		=> lyricist.to_s(),
			'autogen'		=> true
		}

		if ( @review && ! GUI.show_wiki_submit_song_dialog( page_data ) )
			notify( I18N.get( 'wiki.submitsong.cancelled' ) )
			return nil, nil
		end

		if ( Strings.empty?( page_data['lyrics'] ) && ! page_data['instrumental'] )
			notify( I18N.get( 'wiki.submitsong.error.nolyrics' ) )
			return nil, page_data
		end

		page_content = build_song_page(
			page_data['autogen'],
			page_data['artist'],
			page_data['title'],
			page_data['album'],
			page_data['year'],
			page_data['credits'],
			page_data['lyricist'],
			page_data['lyrics']
		)

		summary = "#{page_data['autogen'] ? "autogen. " : ''}song page (#{@@NAME}v#{@@VERSION})"
		if ( submit_page( page_data['url'], page_content, summary ) )
			notify( I18N.get( 'wiki.submitsong.success', page_data['title'], page_data['artist'] ) )
			return page_data['url'], page_data
		else
			notify( I18N.get( 'wiki.submitsong.error', page_data['url'] ) )
			return nil, page_data
		end

	end


	def WikiLyrics.parse_search_results( body, content_matches=false )

		return [] if ( body == nil )

		body.tr_s!( " \n\r\t", ' ' )
		body.gsub!( /\ ?<\/?span[^>]*> ?/, '' )

		results = []

		return results if ( ! body.gsub!( /.*<a name="Article_title_matches">/, '' ) &&
							! body.gsub!( /.*<a name="No_page_title_matches">/, '' ) )

 		if ( ! content_matches )
			body.gsub!( /<a name="Page_text_matches">.*$/, '' )
			body.gsub!( /<a name="No_page_text_matches">.*$/, '' )
		end

		return results if ( ! body.gsub!( /<form id="powersearch" method="get" action="\/Special:Search">.*$/, '' ) )
		body.gsub!( /<\/ol> ?<p>View \(previous .*$/, '' )

		body.split( '<li>' ).each() do |entry|
			if ( ( url_md = /<a href="\/([^"]*)/.match( entry )) && (title_md = /<a .* title="([^"]+)"/.match( entry )) )
				result = { 'url'=>"http://#{site_host()}/index.php?title=#{url_md[1]}", 'title'=>title_md[1] }
				results << result if ( ! content_matches || ! results.include?( result ) )
			end
		end

		return results
	end

	def parse_search_results( body, content_matches=false )
		self.class.parse_search_results( body, content_matches )
	end

	# TODO: add support for nested templates
	def WikiLyrics.parse_template( template )

		begin
			random_token = Strings.random_token()
		end while( template.include?( random_token ) )

		md = /\s*\{\{\s*([^\|]+)\s*\|([^}]+)\}\}\s*/.match( template )
		return nil, nil if ( md == nil )
		name, params = md[1], md[2].gsub( /\[\[([^\[\]]+\|[^\[\]]+)\]\]/ ) { |s| s.gsub( '|', random_token ) }
		params_list = []
		idx = 1
		params.split( /\s*\|\s*/ ).each() do |key_value|
			if ( md = /([a-zA-Z0-9]+)\s*=\s*(.*)\s*/.match( key_value ) ) # unnamed arg
				key, value = md[1], md[2]
			else
				key, value = idx, key_value.strip()
				idx += 1
			end
			value.gsub!( random_token, '|' )
			HTMLEntities.decode!( value )
			params_list.insert( -1, { 'key' => key, 'value' => value } )
		end
		return name.strip(), params_list
	end

	def parse_template( template )
		return self.class.parse_template( template )
	end

	def WikiLyrics.prepare_image_file( image_path, size_limit=153600 )
		4.times() do |trynumb|
			system( 'convert', '-quality', (100-trynumb*10).to_s(), image_path, '/tmp/AlbumArt.jpg' )
			return nil, nil if ( $? != 0 )
			size = FileTest.size?( '/tmp/AlbumArt.jpg' )
			return '/tmp/AlbumArt.jpg', 'image/jpeg' if ( (size ? size : 0) <= size_limit )
		end
		return nil, nil
	end

	def prepare_image_file( image_path, size_limit=153600 )
		return self.class.prepare_image_file( image_path, size_limit )
	end

	def WikiLyrics.parse_tracks( tracks, various_artists='(Various Artists)' )

		tracks_data = { 'genres' => [], 'length' => 0 }

		tracks_data['artist'] = tracks[0]['artist']
		normalized_artist = Strings.normalize( tracks_data['artist'] )
		tracks.each() do |track|
			if ( normalized_artist != Strings.normalize( track['artist'] ) )
				tracks_data['artist'] = various_artists
			end

			genre = track['genre']
			if ( genre && ! tracks_data['genres'].include?( (genre = Strings.downcase!(genre.strip())) ) )
				tracks_data['genres'].insert( -1, genre )
			end

			tracks_data['length'] += track['length'].to_i()
		end

		return tracks_data
	end

	def parse_tracks( tracks, various_artists='(Various Artists)' )
		return self.class.parse_tracks( tracks, various_artists )
	end

	def WikiLyrics.cleanup_title_token( title, downcase=false )
		return cleanup_title_token!( String.new( title ), downcase )
	end

	def cleanup_title_token( title, downcase=false )
		return self.class.cleanup_title_token( title, downcase )
	end

	def cleanup_title_token!( title, downcase=false )
		return self.class.cleanup_title_token!( title, downcase )
	end

	def WikiLyrics.get_sort_name( title )
		return get_sort_name!( String.new( title ) )
	end

	def get_sort_name( title )
		return self.class.get_sort_name( title )
	end

	def WikiLyrics.get_sort_name!( title )

		title.gsub!( /á|ä|à|â|Á|Ä|À|Â/, 'a' )
		title.gsub!( /é|ë|è|ê|É|Ë|È|Ê/, 'e' )
		title.gsub!( /í|ï|ì|î|Í|Ï|Ì|Î/, 'i' )
		title.gsub!( /ó|ö|ò|ô|Ó|Ö|Ò|Ô/, 'o' )
		title.gsub!( /ú|ü|ù|û|Ú|Ü|Ù|Û/, 'u' )

		title.gsub!( /\[[^\]\[]*\]/, '' )
		Strings.titlecase!( title, false, true )

		title.gsub!( /[·\.,;:"`´¿\?¡!\(\)\[\]{}<>#\$\+\*%\^]/, '' )
		title.gsub!( /[\\\/_-]/, ' ' )
		title.gsub!( '&', 'And' )
		title.squeeze!( ' ' )
		title.strip!()

		title.gsub!( /^A /, '' ) # NOTE: only valid for English
		title.gsub!( /^An /, '' )
		title.gsub!( /^The /, '' )
		title.gsub!( /^El /, '' )
		title.gsub!( /^Le /, '' )
		title.gsub!( /^La /, '' )
		title.gsub!( /^L'[AEIOU]/i ) { |s| s.slice(2,1).upcase() }
		title.gsub!( /^Los /, '' )
		title.gsub!( /^Las /, '' )
		title.gsub!( /^Les /, '' )
		title.gsub!( '\'', '' )

		return title
	end

	def get_sort_name!( title )
		return self.class.get_sort_name!( title )
	end

	def WikiLyrics.get_sort_letter( title, normalize_title=false )
		title = get_sort_name( title ) if ( normalize_title )
		title = title.strip()
		if ( title.index( /^[0-9]/ ) == 0 )
			return '0-9'
		else
			# return title.slice( 0, 1 )
			return title.unpack( 'U*' ).slice( 0, 1 ).pack( 'U*' )
		end
	end

	def get_sort_letter( title, normalize_title=false )
		return self.class.get_sort_letter( title, normalize_title )
	end



	# GENERAL FUNCTIONS

	def WikiLyrics.cleanup_article( article, capitalize=true )
		article = article.gsub( '_', ' ' )
		article.strip!()
		article.squeeze!( ' ' )
		Strings.capitalize!( article, false, true ) if ( capitalize )
		return article
	end

	def cleanup_article( article )
		return self.class.cleanup_article( article )
	end

	def WikiLyrics.parse_link( link )

		if ( md = /^ *\[\[([^\|\]]+)\]\] *$/.match( link ) )
			article = cleanup_article( md[1] )
			return article, article if ( ! article.empty?() )
		end

		if ( md = /^ *\[\[([^\|\]]+)\|([^\]]*)\]\] *$/.match( link ) )
			article = cleanup_article( md[1] )
			if ( ! article.empty?() )
				display = cleanup_article( md[2], false )
				return article, display if ( ! display.empty?() )
			end
		end

		return nil, nil
	end

	def parse_link( link )
		return self.class.parse_link( link )
	end

	def WikiLyrics.build_link( article, display=nil )
		return display ? "[[#{article}|#{display}]]" : "[[#{article}]]"
	end

	def build_link( article, display=nil )
		return self.class.build_link( article, display )
	end

	def WikiLyrics.parse_url( url )
		if ( md = /(https?:\/\/[^\/]+\/|)(index.php\?title=|wiki\/|)([^&]+)(&.*|)$/.match( url ) )
			return cleanup_article( CGI.unescape( md[3] ) ) # article title
		else
			return nil
		end
	end

	def parse_url( url )
		return self.class.parse_url( url )
	end

	def WikiLyrics.build_url( article )
		return "http://#{site_host()}/index.php?title=#{CGI.escape( article.gsub( ' ', '_' ) )}"
	end

	def build_url( article )
		return self.class.build_url( article )
	end



	# SONG FUNCTIONS

	def WikiLyrics.parse_song_link( link )
		article, display = parse_link( link )
		return nil, nil if ( article == nil )
		if ( md = /^([^:]+):(.+)$/.match( article ) )
			return md[1], md[2]
		else
			return nil, nil
		end
	end

	def parse_song_link( link )
		return self.class.parse_song_link( link )
	end

	def WikiLyrics.build_song_link( artist, title )
		artist = cleanup_title_token( artist )
		title = cleanup_title_token( title )
		return build_link( "#{artist}:#{title}" )
	end

	def build_song_link( artist, title )
		return self.class.build_song_link( artist, title )
	end

	def WikiLyrics.parse_song_url( url )
		article = parse_url( url )
		return nil, nil if ( article == nil )
		if ( md = /^([^:]+):(.+)$/.match( article ) )
			return md[1], md[2] # artist, song title
		else
			return nil, nil
		end
	end

	def WikiLyrics.build_song_url( artist, title )
		artist = cleanup_title_token( artist )
		title = cleanup_title_token( title )
		return build_url( "#{artist}:#{title}" )
	end

	def build_song_url( artist, title )
		return self.class.build_song_url( artist, title )
	end

	def WikiLyrics.build_song_rawdata_url( artist, title )
		return build_song_url( artist, title ) + '&action=raw&ctype=text/javascript'
	end

	def build_song_rawdata_url( artist, title )
		return self.class.build_song_rawdata_url( artist, title )
	end

	def WikiLyrics.build_song_edit_url( artist, title )
		return build_song_url( artist, title ) + '&action=edit'
	end

	def build_song_edit_url( artist, title )
		return self.class.build_song_edit_url( artist, title )
	end

	def WikiLyrics.build_song_add_url( artist, title, album=nil, year=nil )
		return build_song_url( artist, title ) + '&action=edit'
	end

	def build_song_add_url( artist, title, album=nil, year=nil )
		return self.class.build_song_add_url( artist, title, album, year )
	end

	def WikiLyrics.build_song_search_url( artist, title )
		artist = cleanup_title_token( artist )
		title = cleanup_title_token( title )
		search_string = CGI.escape( "#{artist}:#{title}" )
		return "http://#{site_host()}/index.php?redirs=1&search=#{search_string}&fulltext=Search&limit=500"
	end

	def build_song_search_url( artist, title )
		return self.class.build_song_search_url( artist, title )
	end

	def WikiLyrics.find_song_page_url( artist, title )

		url = build_song_url( artist, title )
		response, url = HTTP.fetch_page_get( url + '&action=raw&ctype=text/javascript' )
		if ( response && response.body().strip() != '' ) # page exists
			return url
		else
			artist = Strings.normalize( cleanup_title_token( artist, false ) )
			title = Strings.normalize( cleanup_title_token( title, false ) )
			normalized_target = "#{artist}:#{title}"

			response, url = HTTP.fetch_page_get( build_song_search_url( artist, title ) )
			return nil if ( response == nil )
			parse_search_results( response.body(), true ).each() do |result|
				normalized_result = result['title'].split( ":" ).each() { |token| Strings.normalize!( token ) }.join( ":" )
				return result['url'] if ( normalized_target == normalized_result )
			end

			return nil
		end

	end

	def find_song_page_url( artist, title )
		return self.class.find_song_page_url( artist, title )
	end



	# ALBUM FUNCTIONS

	def WikiLyrics.parse_album_link( link )
		article, display = parse_link( link )
		return nil, nil, nil if ( article == nil )
		if ( md = /^([^:]+):(.+) \(([0-9]{4,4})\)$/.match( article ) )
			return md[1], md[2], md[3]
		else
			return nil, nil, nil
		end
	end

	def parse_album_link( link )
		return self.class.parse_album_link( link )
	end

	def WikiLyrics.build_album_link( artist, album, year )
		artist = cleanup_title_token( artist )
		album = cleanup_title_token( album )
		return build_link( "#{artist}:#{album} (#{year})" )
	end

	def build_album_link( artist, album, year )
		return self.class.build_album_link( artist, album, year )
	end

	def WikiLyrics.parse_album_url( url )
		article = parse_url( url )
		return nil, nil, nil if ( article == nil )
		if ( md = /^([^:]+):(.+) \(([0-9]{4,4})\)$/.match( article ) )
			return md[1], md[2], md[3]
		else
			return nil, nil, nil
		end
	end

	def parse_album_url( url )
		return self.class.parse_album_url( url )
	end

	def WikiLyrics.build_album_url( artist, album, year )
		artist = cleanup_title_token( artist )
		album = cleanup_title_token( album )
		return build_url( "#{artist}:#{album} (#{year})" )
	end

	def build_album_url( artist, album, year )
		return self.class.build_album_url( artist, album, year )
	end

	def WikiLyrics.build_album_rawdata_url( artist, album, year )
		return build_album_url( artist, album, year ) + '&action=raw&ctype=text/javascript'
	end

	def build_album_rawdata_url( artist, album, year )
		return self.class.build_album_rawdata_url( artist, album, year )
	end

	def WikiLyrics.build_album_edit_url( artist, album, year )
		return build_album_url( artist, album, year ) + '&action=edit'
	end

	def build_album_edit_url( artist, album, year )
		return self.class.build_album_edit_url( artist, album, year )
	end

	def WikiLyrics.build_album_search_url( artist, album, year )
		artist = cleanup_title_token( artist )
		album = cleanup_title_token( album )
		search_string = CGI.escape( "#{artist}:#{album} (#{year})" )
		return "http://#{site_host()}/index.php?redirs=1&search=#{search_string}&fulltext=Search&limit=500"
	end

	def build_album_search_url( artist, album, year )
		return self.class.build_album_search_url( artist, album, year )
	end

	def WikiLyrics.find_album_page_url( artist, album, year )

		url = build_album_url( artist, album, year )
		response, url = HTTP.fetch_page_get( url + '&action=raw&ctype=text/javascript' )
		if ( response && response.body().strip() != '' ) # page exists
			return url
		else
			artist = Strings.normalize( cleanup_title_token( artist, false ) )
			album = Strings.normalize( cleanup_title_token( album, false ) )
			normalized_target = "#{artist}:#{album} (#{year})"

			response, url = HTTP.fetch_page_get( build_album_search_url( artist, album, year ) )
			return nil if ( response == nil || response.body() == nil )
			parse_search_results( response.body(), true ).each() do |result|
				normalized_result = result['title'].split( ":" ).each() { |token| Strings.normalize!( token ) }.join( ":" )
				return result['url'] if ( normalized_target == normalized_result )
			end
			return nil
		end

	end

	def find_album_page_url( artist, album, year )
		return self.class.find_album_page_url( artist, album, year )
	end


	# ALBUM ART FUNCTIONS

	def build_album_art_description( artist, album, year )
		return self.class.build_album_art_description( artist, album, year )
	end

	def build_album_art_name( artist, album, year, extension='jpg' )
		return self.class.build_album_art_name( artist, album, year, extension )
	end

	def find_album_art_name( artist, album, year )
		return self.class.find_album_art_name( artist, album, year )
	end

	def WikiLyrics.find_album_art_url( artist, album, year )
		if ( album_art_name = find_album_art_name( artist, album, year ) )
			album_art_name.gsub!( ' ', '_' )
			return "http://#{site_host()}/index.php?title=Image:#{CGI.escape(album_art_name)}"
		else
			return nil
		end
	end

	def find_album_art_url( artist, album, year )
		return self.class.find_album_art_url( artist, album, year )
	end
end
